@@ -1,6 +1,6 @@
 /* dirfd.c -- return the file descriptor associated with an open DIR*
 
-   Copyright (C) 2001, 2006 Free Software Foundation, Inc.
+   Copyright (C) 2001, 2006, 2008-2009 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published by
@@ -19,10 +19,14 @@
 
 #include <config.h>
 
-#include "dirfd.h"
+#include <dirent.h>
+#include <errno.h>
 
 int
 dirfd (DIR const *dir_p)
 {
-  return DIR_TO_FD (dir_p);
+  int fd = DIR_TO_FD (dir_p);
+  if (fd == -1)
+    errno = ENOTSUP;
+  return fd;
 }
