@@ -22,7 +22,6 @@
 *****/
 
 #include "config.h"
-#include "libmissing.h"
 
 #include <stdio.h>
 #include <string.h>
@@ -117,7 +116,7 @@ static int parse_value(value_container_t *vcont, const char *line)
                         i = 0;
                         str++;
 
-                        while ( isdigit((int) *str) && i < sizeof(num) )
+                        while ( isdigit((int) *str) && i < (sizeof(num) - 1) )
                                 num[i++] = *str++;
 
                         if ( ! i )
