@@ -6,7 +6,7 @@
 * This file is part of the Prelude-LML program.
 *
 * This program is free software; you can redistribute it and/or modify
-* it under the terms of the GNU General Public License as published by 
+* it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2, or (at your option)
 * any later version.
 *
@@ -22,7 +22,6 @@
 *****/
 
 #include "config.h"
-#include "libmissing.h"
 
 #include <stdio.h>
 #include <stdarg.h>
@@ -54,12 +53,22 @@
 
 
 
+typedef struct {
+        prelude_list_t list;
+        prelude_bool_t force;
+        idmef_path_t *path;
+        idmef_value_t *value;
+} lml_format_idmef_t;
+
+
 struct lml_log_format {
         int refcount;
         char *name;
         char *ts_fmt;
         pcre *prefix_regex;
         pcre_extra *prefix_regex_extra;
+
+        prelude_list_t idmef_list;
 };
 
 
@@ -71,7 +80,7 @@ struct lml_log_format_container {
 
 struct lml_log_source {
         prelude_list_t list;
-        
+
         char *name;
         regex_list_t *rlist;
 
@@ -92,13 +101,13 @@ static PRELUDE_LIST(source_list);
  * Log format stuff
  */
 static void lml_log_format_destroy(lml_log_format_t *format)
-{        
+{
         if ( --format->refcount != 0 )
                 return;
- 
+
         if ( format->ts_fmt )
                 free(format->ts_fmt);
-        
+
         if ( format->prefix_regex)
                 free(format->prefix_regex);
 
@@ -123,13 +132,13 @@ lml_log_format_t *lml_log_format_new(const char *name)
                 prelude_log(PRELUDE_LOG_ERR, "memory exhausted.\n");
                 return NULL;
         }
-        
+
         new->name = strdup(name);
         if ( ! new->name ) {
                 free(new);
                 return NULL;
         }
-        
+
         if ( lml_log_format_set_ts_fmt(new, SYSLOG_TS_FMT) < 0 ) {
                 prelude_log(PRELUDE_LOG_WARN, "failed to set log timestamp format.\n");
                 return NULL;
@@ -139,7 +148,8 @@ lml_log_format_t *lml_log_format_new(const char *name)
                 prelude_log(PRELUDE_LOG_WARN, "failed to set log message prefix.\n");
                 return NULL;
         }
-        
+
+        prelude_list_init(&new->idmef_list);
         return new;
 }
 
@@ -153,13 +163,13 @@ const char *lml_log_format_get_name(lml_log_format_t *lf)
 
 
 int lml_log_format_set_prefix_regex(lml_log_format_t *ls, const char *regex)
-{       
+{
         int erroffset;
         const char *errptr;
 
         if ( ls->prefix_regex )
                 free(ls->prefix_regex);
-        
+
         ls->prefix_regex = pcre_compile(regex, 0, &errptr, &erroffset, NULL);
         if ( ! ls->prefix_regex ) {
                 prelude_log(PRELUDE_LOG_WARN, "Unable to compile regex: %s : %s.\n", regex, errptr);
@@ -179,10 +189,10 @@ int lml_log_format_set_prefix_regex(lml_log_format_t *ls, const char *regex)
 
 
 int lml_log_format_set_ts_fmt(lml_log_format_t *ls, const char *fmt)
-{        
+{
         if ( ls->ts_fmt )
                 free(ls->ts_fmt);
-        
+
         ls->ts_fmt = strdup(fmt);
         if ( ! ls->ts_fmt ) {
                 prelude_log(PRELUDE_LOG_ERR, "memory exhausted.\n");
@@ -195,6 +205,81 @@ int lml_log_format_set_ts_fmt(lml_log_format_t *ls, const char *fmt)
 
 
 
+
+int lml_log_format_set_idmef(lml_log_format_t *format, const char *idmef_s, prelude_bool_t force)
+{
+        int ret;
+        size_t len;
+        char *idmef, tmp;
+        lml_format_idmef_t *entry;
+
+        len = strcspn(idmef_s, " =");
+        if ( len == 0 )
+                return -1;
+
+        idmef = strdup(idmef_s);
+        if ( ! idmef )
+                return -1;
+
+        entry = malloc(sizeof(*entry));
+        if ( ! entry ) {
+                free(idmef);
+                return -1;
+        }
+
+        entry->force = force;
+
+        tmp = idmef[len];
+        idmef[len] = 0;
+
+        ret = idmef_path_new_fast(&entry->path, idmef);
+        if ( ret < 0 ) {
+                free(idmef);
+                free(entry);
+                return ret;
+        }
+
+        idmef[len] = tmp;
+
+        ret = idmef_value_new_from_path(&entry->value, entry->path, idmef + len + strspn(idmef + len, " ="));
+        if ( ret < 0 ) {
+                free(idmef);
+                idmef_path_destroy(entry->path);
+                free(entry);
+                return ret;
+        }
+
+        free(idmef);
+        prelude_list_add_tail(&format->idmef_list, &entry->list);
+
+        return 0;
+}
+
+
+
+void lml_log_format_apply_idmef(const lml_log_format_t *format, idmef_message_t *idmef)
+{
+        int ret;
+        prelude_list_t *tmp;
+        idmef_value_t *value;
+        lml_format_idmef_t *entry;
+
+        prelude_list_for_each(&format->idmef_list, tmp) {
+                entry = prelude_list_entry(tmp, lml_format_idmef_t, list);
+
+                if ( ! entry->force ) {
+                        ret = idmef_path_get(entry->path, idmef, &value);
+                        if ( ret > 0 ) {
+                                idmef_value_destroy(value);
+                                continue;
+                        }
+                }
+
+                idmef_path_set(entry->path, idmef, entry->value);
+        }
+}
+
+
 const char *lml_log_format_get_ts_fmt(const lml_log_format_t *source)
 {
         return source->ts_fmt;
@@ -228,10 +313,10 @@ static lml_log_source_t *search_source(const char *name)
 {
         prelude_list_t *tmp;
         lml_log_source_t *ls;
-        
+
         prelude_list_for_each(&source_list, tmp) {
                 ls = prelude_linked_object_get_object(tmp);
-                
+
                 if ( strcmp(lml_log_source_get_name(ls), name) == 0 )
                         return ls;
         }
@@ -243,7 +328,7 @@ static lml_log_source_t *search_source(const char *name)
 static int source_set_format(lml_log_source_t *ls, lml_log_format_t *format)
 {
         lml_log_format_container_t *fc;
-        
+
         fc = malloc(sizeof(*fc));
         if ( ! fc )
                 return -1;
@@ -263,17 +348,17 @@ regex_list_t *lml_log_source_get_regex_list(lml_log_source_t *ls)
 
 
 
-int lml_log_source_set_name(lml_log_source_t *ls, const char *name) 
+int lml_log_source_set_name(lml_log_source_t *ls, const char *name)
 {
         if ( ls->name )
                 free(ls->name);
-        
+
         ls->name = strdup(name);
         if ( ! ls->name ) {
                 prelude_log(PRELUDE_LOG_ERR, "memory exhausted.\n");
                 return -1;
         }
-        
+
         return 0;
 }
 
@@ -287,23 +372,23 @@ int lml_log_source_new(lml_log_source_t **ls, lml_log_format_t *format, const ch
                 ret = source_set_format(*ls, format);
                 if ( ret < 0 )
                         return -1;
-                
+
                 return 1;
         }
-        
+
         *ls = calloc(1, sizeof(**ls));
         if ( ! *ls ) {
                 prelude_log(PRELUDE_LOG_ERR, "memory exhausted.\n");
                 return -1;
         }
-        
+
         (*ls)->name = strdup(name);
         if ( ! (*ls)->name) {
                 prelude_log(PRELUDE_LOG_ERR, "memory exhausted.\n");
                 free(*ls);
                 return -1;
         }
-        
+
         (*ls)->rlist = regex_init(name);
         if ( ! (*ls)->rlist ) {
                 free((*ls)->name);
@@ -311,7 +396,7 @@ int lml_log_source_new(lml_log_source_t **ls, lml_log_format_t *format, const ch
                 return -1;
         }
 
-        (*ls)->warning_limit = config.warning_limit;        
+        (*ls)->warning_limit = config.warning_limit;
         prelude_list_init(&(*ls)->format_list);
 
         ret = source_set_format(*ls, format);
@@ -319,14 +404,14 @@ int lml_log_source_new(lml_log_source_t **ls, lml_log_format_t *format, const ch
                 return ret;
 
         prelude_list_add_tail(&source_list, &(*ls)->list);
-        
+
         return 0;
 }
 
 
 
 const char *lml_log_source_get_name(const lml_log_source_t *ls)
-{        
+{
         return ls->name;
 }
 
@@ -336,7 +421,7 @@ void lml_log_source_destroy(lml_log_source_t *source)
 {
         prelude_list_t *tmp, *bkp;
         lml_log_format_container_t *fc;
-        
+
         prelude_list_for_each_safe(&source->format_list, tmp, bkp) {
                 fc = prelude_linked_object_get_object(tmp);
                 prelude_linked_object_del((prelude_linked_object_t *) fc);
@@ -344,13 +429,13 @@ void lml_log_source_destroy(lml_log_source_t *source)
                 lml_log_format_destroy(fc->format);
                 free(fc);
         }
-        
+
         if ( source->rlist )
                 regex_destroy(source->rlist);
-        
+
         if ( source->name )
                 free(source->name);
-        
+
         free(source);
 }
 
@@ -365,7 +450,7 @@ void lml_log_source_warning(lml_log_source_t *ls, const char *fmt, ...)
          */
         if ( ls->warning_limit > 0 && ls->warning_count == ls->warning_limit ) {
                 ls->warning_count++;
-                
+
                 prelude_log(PRELUDE_LOG_WARN, "Limit of %d errors for source %s reached. Further errors will be supressed.\n",
                             ls->warning_limit, lml_log_source_get_name(ls));
                 return;
@@ -375,7 +460,7 @@ void lml_log_source_warning(lml_log_source_t *ls, const char *fmt, ...)
                 return;
 
         ls->warning_count++;
-        
+
         va_start(ap, fmt);
         prelude_log_v(PRELUDE_LOG_WARN, fmt, ap);
         va_end(ap);
@@ -387,3 +472,4 @@ prelude_list_t *lml_log_source_get_format_list(lml_log_source_t *source)
 {
         return &source->format_list;
 }
+
